#include <stdio.h>
#include <unistd.h>
#include <pthread.h>

#include "tuya_ipc_service_only_init.h"

#include "ty_cJSON.h"
#include "tuya_app_config.h"
#include "tuya_ipc_media.h"
#include "uni_log.h"
#include "user_ipc_media_demo.h"
#include "user_ipc_p2p_demo.h"
#include "user_ipc_notify_demo.h"

/**
 * {
 *     "tuya": {
 *         "ipc": {
 *             "lowpower": "0",
 *             "media_path" : "./",
 *             "storage_path" : "./"
 *         }
 *     }
 * }
 */

typedef struct
{
    BOOL_T is_lowpower;
    CHAR_T media_path[128];
    CHAR_T storage_path[128];

    BOOL_T is_onlined;
}TUYA_DEMO_IPC_T;

STATIC TUYA_DEMO_IPC_T sg_demo_ipc = {0};

STATIC OPERATE_RET __ipc_parse_cfg(VOID)
{
    TUYA_PLATEFORM_S *cfg = tuya_get_config();
    ty_cJSON *root = NULL;
    ty_cJSON *json_ipc = NULL;
    ty_cJSON *json_lowpower = NULL;
    ty_cJSON *json_media_path = NULL;
    ty_cJSON *json_storage_path = NULL;

    root = ty_cJSON_GetObjectItem(cfg->input, "tuya");
    TUYA_CHECK_NULL_RETURN(root, NULL);

    json_ipc = ty_cJSON_GetObjectItem(root, "ipc");
    TUYA_CHECK_NULL_RETURN(json_ipc, NULL);

    json_lowpower = ty_cJSON_GetObjectItem(json_ipc, "lowpower");
    if(json_lowpower != NULL) {
        sg_demo_ipc.is_lowpower = json_lowpower->valueint;
    }    

    json_media_path = ty_cJSON_GetObjectItem(json_ipc, "media_path");
    TUYA_CHECK_NULL_RETURN(json_media_path, NULL);
    strncpy(sg_demo_ipc.media_path, json_media_path->valuestring, 128);

    json_storage_path = ty_cJSON_GetObjectItem(json_ipc, "storage_path");
    TUYA_CHECK_NULL_RETURN(json_storage_path, NULL);
    strncpy(sg_demo_ipc.storage_path, json_storage_path->valuestring, 128);

    return OPRT_OK;
}

STATIC VOID demo_ipc_online_work()
{
    //接口不要阻塞。仅设置标位即可。
    sg_demo_ipc.is_onlined = TRUE;
    
    return;
}

VOID IPC_APP_run_thread()
{
    while(sg_demo_ipc.is_onlined != TRUE) {
        usleep(100*1000);
    }

    /**
     * 流媒体业务初始化
    */
    IPC_APP_Media_Stream_Init();

    /**
     * SD卡录像初始化
    */
    TUYA_APP_Init_Stream_Storage(sg_demo_ipc.storage_path);

    /**
     * 云存储初始化
    */
    TUYA_APP_Enable_CloudStorage();

    /**
     * 事件报警初始化
    */
    IPC_APP_event_module_init();

    /**
     * 重要！！！IPC大部分业务有能力值标位，请在所有业务初始化后触发下面接口，上报能力值。
    */
    tuya_ipc_upload_skills();
}

/****************************/
OPERATE_RET user_ipc_demo_init()
{
    OPERATE_RET rt = OPRT_OK;

    TUYA_CALL_ERR_RETURN(__ipc_parse_cfg());

    IPC_SVC_PARAM_T ipc_param = {0};
    ipc_param.is_lowpower = sg_demo_ipc.is_lowpower;
    ipc_param.on_ipc_online = demo_ipc_online_work;

    sg_demo_ipc.is_onlined = FALSE;
    TUYA_CALL_ERR_RETURN(tuya_imm_service_init(&ipc_param));

    pthread_t ipc_run_thread;
    pthread_create(&ipc_run_thread, NULL, IPC_APP_run_thread, NULL);
    pthread_detach(ipc_run_thread);

    return rt;
}

OPERATE_RET user_ipc_demo_start()
{
    OPERATE_RET rt = OPRT_OK;
    IPC_MEDIA_INFO_T media_info = {0};

    TUYA_CALL_ERR_RETURN(IPC_APP_media_task_start(sg_demo_ipc.media_path));
    TUYA_CALL_ERR_RETURN(TUYA_APP_Enable_Motion_Detect());
    TUYA_CALL_ERR_RETURN(IPC_APP_get_media_info(&media_info));
    TUYA_CALL_ERR_RETURN(TUYA_APP_Init_Ring_Buffer(&media_info));
    IPC_APP_Media_Adapter_Init();
    tuya_ipc_ring_buffer_adapter_register_media_source();
    return OPRT_OK;
}

VOID IPC_APP_simulation()
{
    /* Manual simulation of related functions */
    char test_input[64] = {0};

    while(1)
    {
        scanf("%s",test_input);
        /* Simulation of the start of motion detection events */
        if(0 == strcmp(test_input,"start"))
        {
            IPC_APP_set_motion_status(1);
        }
        /* Simulation of the end of event */
        else if(0 == strcmp(test_input,"stop"))
        {
            IPC_APP_set_motion_status(0);
        }
        /* Simulation of getting device's activation status */
        else if(0 == strcmp(test_input,"status"))
        {
            IPC_REGISTER_STATUS_E status = tuya_ipc_get_register_status();
            PR_INFO("current register status %d[0:unregistered 1:registered 2:activated]\n",status);
        }

        usleep(100*1000);
    }

    return;
}