#include <stdio.h>
#include <unistd.h>
#include <pthread.h>

#include "tuya_cloud_types.h"
#include "ty_cJSON.h"
#include "tuya_app_config.h"
#include "tuya_ipc_media.h"
#include "tuya_ring_buffer.h"
#include "tuya_ipc_media_adapter.h"
#include "uni_log.h"

/*
---------------------------------------------------------------------------------
code related media info
---------------------------------------------------------------------------------
*/

STATIC IPC_MEDIA_INFO_T sg_media_info = {0};

STATIC OPERATE_RET IPC_APP_media_info_init()
{
	/*media info (essential)*/
    /* main stream(HD), video configuration*/
    /* NOTE
    FIRST:If the main stream supports multiple video stream configurations, set each item to the upper limit of the allowed configuration.
    SECOND:E_IPC_STREAM_VIDEO_MAIN must exist.It is the data source of SDK.
    please close the E_IPC_STREAM_VIDEO_SUB for only one stream*/
    sg_media_info.stream_enable[E_IPC_STREAM_VIDEO_MAIN] = TRUE;    /* Whether to enable local HD video streaming */
    sg_media_info.video_fps[E_IPC_STREAM_VIDEO_MAIN] = 30;  /* FPS */
    sg_media_info.video_gop[E_IPC_STREAM_VIDEO_MAIN] = 30;  /* GOP */
    sg_media_info.video_bitrate[E_IPC_STREAM_VIDEO_MAIN] = TUYA_VIDEO_BITRATE_1M; /* Rate limit */
    sg_media_info.video_width[E_IPC_STREAM_VIDEO_MAIN] = 640; /* Single frame resolution of width*/
    sg_media_info.video_height[E_IPC_STREAM_VIDEO_MAIN] = 360;/* Single frame resolution of height */
    sg_media_info.video_freq[E_IPC_STREAM_VIDEO_MAIN] = 90000; /* Clock frequency */
    sg_media_info.video_codec[E_IPC_STREAM_VIDEO_MAIN] = TUYA_CODEC_VIDEO_H264; /* Encoding format */

    /* substream(HD), video configuration */
    /* Please note that if the substream supports multiple video stream configurations, please set each item to the upper limit of the allowed configuration. */
    sg_media_info.stream_enable[E_IPC_STREAM_VIDEO_SUB] = TRUE;     /* Whether to enable local SD video stream */
    sg_media_info.video_fps[E_IPC_STREAM_VIDEO_SUB] = 30;  /* FPS */
    sg_media_info.video_gop[E_IPC_STREAM_VIDEO_SUB] = 30;  /* GOP */
    sg_media_info.video_bitrate[E_IPC_STREAM_VIDEO_SUB] = TUYA_VIDEO_BITRATE_512K; /* Rate limit */
    sg_media_info.video_width[E_IPC_STREAM_VIDEO_SUB] = 640; /* Single frame resolution of width */
    sg_media_info.video_height[E_IPC_STREAM_VIDEO_SUB] = 360;/* Single frame resolution of height */
    sg_media_info.video_freq[E_IPC_STREAM_VIDEO_SUB] = 90000; /* Clock frequency */
    sg_media_info.video_codec[E_IPC_STREAM_VIDEO_SUB] = TUYA_CODEC_VIDEO_H264; /* Encoding format */

    /* Audio stream configuration.
    Note: The internal P2P preview, cloud storage, and local storage of the SDK are all use E_IPC_STREAM_AUDIO_MAIN data. */
    sg_media_info.stream_enable[E_IPC_STREAM_AUDIO_MAIN] = TRUE;         /* Whether to enable local sound collection */
    sg_media_info.audio_codec[E_IPC_STREAM_AUDIO_MAIN] = TUYA_CODEC_AUDIO_PCM;/* Encoding format */
    sg_media_info.audio_sample [E_IPC_STREAM_AUDIO_MAIN]= TUYA_AUDIO_SAMPLE_8K;/* Sampling Rate */
    sg_media_info.audio_databits [E_IPC_STREAM_AUDIO_MAIN]= TUYA_AUDIO_DATABITS_16;/* Bit width */
    sg_media_info.audio_channel[E_IPC_STREAM_AUDIO_MAIN]= TUYA_AUDIO_CHANNEL_MONO;/* channel */
    sg_media_info.audio_fps[E_IPC_STREAM_AUDIO_MAIN] = 25;/* Fragments per second */    

    return OPRT_OK;
}


OPERATE_RET IPC_APP_get_media_info(IPC_MEDIA_INFO_T* media_info)
{
    memcpy(media_info, &sg_media_info, SIZEOF(IPC_MEDIA_INFO_T));   

    return OPRT_OK;
}


/*
---------------------------------------------------------------------------------
code related MEDIA task
---------------------------------------------------------------------------------
*/
STATIC CHAR_T sg_media_path[128] = {0};
#define AUDIO_FRAME_SIZE 640
#define AUDIO_FPS 25
#define VIDEO_BUF_SIZE	(1024 * 400) //Maximum frame
#define MAX_SNAPSHOT_BUFFER_SIZE            (100*1024)  //in BYTE

int IPC_APP_get_snapshot(char *snap_addr, int *snap_size)
{
    if (snap_addr == NULL || snap_size == NULL)
    {
        PR_ERR("get snapshot failed, param error\n");
        return -1;
    }

    //we use file to simulate
    char snapfile[128] = {0};
    *snap_size = 0;
    PR_INFO("get snapshot\n");
    snprintf(snapfile, sizeof(snapfile), "%s/resource/media/demo_snapshot.jpg",sg_media_path);
    FILE*fp = fopen(snapfile,"r+");
    if(NULL == fp)
    {
        PR_ERR("fail to open snap.jpg\n");
        return -1;
    }
    fseek(fp,0,SEEK_END);
    *snap_size = ftell(fp);
    if(*snap_size < 100*1024)
    {
        fseek(fp,0,SEEK_SET);
        fread(snap_addr,*snap_size,1,fp);
    }
    fclose(fp);
    return 0;
}

/* This is for demo only. Should be replace with real PCM/AAC/G711 encoder output */
static void *thread_live_audio(void *arg)
{
    char fullpath[128] = {0};
    sprintf(fullpath, "%s/resource/media/demo_audio.raw", sg_media_path);

    FILE *aFp = fopen(fullpath, "rb");
    if(aFp == NULL)
    {
        PR_ERR("can't read live audio file %s\n",fullpath);
        pthread_exit(0);
    }
    RING_BUFFER_USER_HANDLE_T a_handle = NULL ;
    char audioBuf[AUDIO_FRAME_SIZE];
    MEDIA_FRAME_T pcm_frame = {0};
    pcm_frame.type = E_AUDIO_FRAME;

    while(1)
    {
        if(a_handle == NULL) {
        	a_handle = tuya_ipc_ring_buffer_open(0, 0, E_IPC_STREAM_AUDIO_MAIN, E_RBUF_WRITE);
            if (a_handle == NULL) {
                PR_ERR("tuya_ipc_ring_buffer_open err  ");
                sleep(1);
                continue;
            }
            PR_DEBUG("tuya_ipc_ring_buffer_open success  ");
        }
        int size = fread(audioBuf, 1, AUDIO_FRAME_SIZE, aFp);
        if(size < AUDIO_FRAME_SIZE)
        {
            rewind(aFp);
            continue;
        }
        int frameRate = AUDIO_FPS;
        int sleepTick = 1000000/frameRate;
        static UINT64_T pts = 0;
        pts += sleepTick;
        pcm_frame.size = size;
        pcm_frame.p_buf = audioBuf;
        pcm_frame.pts = pts;
        TUYA_APP_Put_Frame(a_handle,&pcm_frame);

        usleep(sleepTick);
    }
    if(a_handle) {
	    tuya_ipc_ring_buffer_close(a_handle);   
	    a_handle = NULL ;
    }
    pthread_exit(0);
}

/* This is for demo only. Should be replace with real H264 encoder output */
static int read_one_frame_from_demo_video_file(unsigned char *pVideoBuf,unsigned int offset,unsigned int BufSize,unsigned int *IskeyFrame,unsigned int 
                                        *FramLen,unsigned int *Frame_start)
{
    int pos = 0;
    int bNeedCal = 0;
    unsigned char NalType=0;
    int idx=0;
    if(BufSize<=5)
    {
        PR_ERR("bufSize is too small\n");
        return -1;
    }
    for(pos=0;pos <= BufSize-5;pos++)
    {
        if(pVideoBuf[pos]==0x00
            &&pVideoBuf[pos+1]==0x00
            &&pVideoBuf[pos+2]==0x00
            &&pVideoBuf[pos+3]==0x01)
        {
            NalType = pVideoBuf[pos + 4] & 0x1f;
            if(NalType == 0x7)
            {
                if(bNeedCal==1)
                {
                    *FramLen=pos-idx;
                    return 0;
                }
                
                *IskeyFrame = 1;
                *Frame_start = offset+pos;
                bNeedCal=1;
                idx=pos;
            }
            else if(NalType ==0x1)                
            {
               if(bNeedCal)
               {
                  *FramLen=pos-idx;
                  return 0;
               }
               *Frame_start=offset+pos;
               *IskeyFrame = 0;
               idx = pos;
               bNeedCal=1;
            }
        }
    }

    *FramLen=BufSize;
    
    return 0;
}

static void *thread_live_video(void *arg)
{
    char raw_fullpath[128] = {0};
    unsigned int FrameLen=0,Frame_start=0;
    unsigned int offset=0;
    unsigned int IsKeyFrame=0;
    unsigned char *pVideoBuf = NULL;
    sprintf(raw_fullpath, "%s/resource/media/demo_video.264", sg_media_path);

    PR_DEBUG("start live video using %s",raw_fullpath);

    FILE *streamBin_fp = fopen(raw_fullpath, "rb");
    if((streamBin_fp == NULL))
    {
        PR_ERR("can't read live video file %s\n",raw_fullpath);
        pthread_exit(0);
    }
    fseek(streamBin_fp, 0, SEEK_END);
    UINT_T file_size = ftell(streamBin_fp);
    fseek(streamBin_fp, 0, SEEK_SET);
    pVideoBuf = malloc(file_size);
    fread(pVideoBuf, 1, file_size, streamBin_fp);

    RING_BUFFER_USER_HANDLE_T v_handle_main = NULL ;
    RING_BUFFER_USER_HANDLE_T v_handle_sub = NULL ;
    MEDIA_FRAME_T video_frame = {0};
    
    while(1)
    {
		if(v_handle_main == NULL) {
			v_handle_main = tuya_ipc_ring_buffer_open(0, 0, E_IPC_STREAM_VIDEO_MAIN, E_RBUF_WRITE);
		    if (v_handle_main == NULL) {
		        PR_ERR("tuya_ipc_ring_buffer_open err  ");
		        sleep(1);
		        continue;
		    }
		    PR_DEBUG("tuya_ipc_ring_buffer_open success  ");
		}
		if(v_handle_sub == NULL) {
		    v_handle_sub = tuya_ipc_ring_buffer_open(0, 0, E_IPC_STREAM_VIDEO_SUB, E_RBUF_WRITE);
		    if (v_handle_sub == NULL) {
		         PR_ERR("tuya_ipc_ring_buffer_open err  ");
		        sleep(1);
		        continue;
		    }
		    PR_DEBUG("tuya_ipc_ring_buffer_open success  ");
		}

        offset=Frame_start+FrameLen;
        if(offset>=file_size)
        {
            offset=0;
        }
        read_one_frame_from_demo_video_file(pVideoBuf+offset,offset,file_size-offset,&IsKeyFrame,&FrameLen,&Frame_start);
        //Note: For I frame of H264, SPS/PPS/SEI/IDR should be combined within one frame, and the NALU separator should NOT be deleted.
        if(IsKeyFrame==1)
        {
            video_frame.type = E_VIDEO_I_FRAME;
            video_frame.size = FrameLen;
        }
        else
        {
            video_frame.type = E_VIDEO_PB_FRAME;
            video_frame.size = FrameLen;
        }

        int frameRate = 30;
        int sleepTick = 1000000/frameRate;
        static UINT64_T pts = 0;
        pts += sleepTick;

        video_frame.p_buf = pVideoBuf+Frame_start;
        video_frame.pts = pts;
        /* Send HD video data to the SDK */
        TUYA_APP_Put_Frame(v_handle_main, &video_frame);
        /* Send SD video data to the SDK */
        TUYA_APP_Put_Frame(v_handle_sub, &video_frame);
        usleep(sleepTick);
    }
    if(v_handle_main) {
        tuya_ipc_ring_buffer_close(v_handle_main);
        v_handle_main = NULL ;
    }

    if(v_handle_sub) {
        tuya_ipc_ring_buffer_close(v_handle_sub);
        v_handle_sub = NULL ;
    }
    
    pthread_exit(0);
}

OPERATE_RET IPC_APP_media_task_start(CHAR_T *media_path)
{
    strncpy(sg_media_path, media_path, 128);
    IPC_APP_media_info_init();

    pthread_t h264_output_thread;
    pthread_create(&h264_output_thread, NULL, thread_live_video, NULL);
    pthread_detach(h264_output_thread);

    pthread_t pcm_output_thread;
    pthread_create(&pcm_output_thread, NULL, thread_live_audio, NULL);
    pthread_detach(pcm_output_thread);
   
    return OPRT_OK;
}


VOID __TUYA_APP_rev_audio_cb(IN INT_T device, IN INT_T channel,IN CONST MEDIA_AUDIO_FRAME_T *p_audio_frame, IN CONST UINT_T frame_no)
{
    MEDIA_FRAME_T audio_frame = {0};
    audio_frame.p_buf = p_audio_frame->p_audio_buf;
    audio_frame.size = p_audio_frame->buf_len;

    PR_INFO("Rev Audio. size:[%u] audio_codec:[%d] audio_sample:[%d] audio_databits:[%d] audio_channel:[%d]\n",p_audio_frame->buf_len,
             p_audio_frame->audio_codec, p_audio_frame->audio_sample, p_audio_frame->audio_databits, p_audio_frame->audio_channel);

//    TUYA_APP_Rev_Audio_CB( &audio_frame, TUYA_AUDIO_SAMPLE_8K, TUYA_AUDIO_DATABITS_16, TUYA_AUDIO_CHANNEL_MONO);

    return;
}


VOID __TUYA_APP_rev_video_cb(IN INT_T device, IN INT_T channel, IN CONST MEDIA_VIDEO_FRAME_T *p_video_frame)
{
    PR_INFO("Rev video. size:[%u] video_codec:[%d] video_frame_type:[%d]\n", p_video_frame->buf_len, p_video_frame->video_codec, p_video_frame->video_frame_type);

    return;
}

OPERATE_RET TUYA_APP_get_snapshot_cb(IN INT_T device, IN INT_T channel, OUT CHAR_T *snap_addr, OUT INT_T *snap_size)
{
    // in this demo ignore device and channel parameter
    return IPC_APP_get_snapshot(snap_addr, snap_size);
}

/* Callback of talkback mode,turn on or off speaker hardware*/
VOID TUYA_APP_Enable_Speaker_CB(BOOL_T enabled)
{
    PR_INFO("enable speaker %d \r\n", enabled);
    //TODO
    /* Developers need to turn on or off speaker hardware operations. 
    If IPC hardware features do not need to be explicitly turned on, the function can be left blank. */
}

VOID IPC_APP_Media_Adapter_Init()
{
    TUYA_IPC_MEDIA_ADAPTER_VAR_T media_var;
    media_var.get_snapshot_cb = TUYA_APP_get_snapshot_cb;
    media_var.on_recv_audio_cb = __TUYA_APP_rev_audio_cb;
    media_var.on_recv_video_cb = __TUYA_APP_rev_video_cb;
    media_var.on_recv_file_cb = NULL;

    //单位MB。0使用默认值。若设置为n，则为最大提供n MB内存用于流媒体传输。一路P2P连接占用1MB，一路WebRTC占用1.5MB
    media_var.available_media_memory = 0; 
    
    tuya_ipc_media_adapter_init(&media_var);
    
    DEVICE_MEDIA_INFO_T device_media_info = { 0 };
    memcpy(&device_media_info.av_encode_info, &sg_media_info,sizeof(IPC_MEDIA_INFO_T));
    
    device_media_info.audio_decode_info.enable = 1;
    device_media_info.audio_decode_info.audio_codec = TUYA_CODEC_AUDIO_G711U;
    device_media_info.audio_decode_info.audio_sample = TUYA_AUDIO_SAMPLE_8K;
    device_media_info.audio_decode_info.audio_databits = TUYA_AUDIO_DATABITS_16;
    device_media_info.audio_decode_info.audio_channel = TUYA_AUDIO_CHANNEL_MONO;
    
    device_media_info.max_pic_len = MAX_SNAPSHOT_BUFFER_SIZE;

    tuya_ipc_media_adapter_set_media_info(0, 0, device_media_info);

    return;
}